import 'package:flutter/material.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:latlong2/latlong.dart';
import 'package:gui_app/const/colors.dart';
import 'package:gui_app/const/config.dart';
import 'package:gui_app/const/dimens.dart';

class OfflineMap extends ConsumerStatefulWidget {
  const OfflineMap({super.key});
  @override
  ConsumerState<OfflineMap> createState() {
    return _MapState();
  }
}

class _MapState extends ConsumerState<OfflineMap>
    with TickerProviderStateMixin {
  final _mapController = MapController();
  late Map<String, double> pinSize;
  late Map<String, double> latitude;
  late Map<String, double> longitude;

  @override
  void initState() {
    super.initState();
    pinSize = {
      'Sapporo': mapPinDefaultSize,
      'Tokyo': mapPinDefaultSize,
      'Osaka': mapPinDefaultSize,
      'Asahikawa': mapPinDefaultSize,
      'Fukushima': mapPinDefaultSize,
    };
    latitude = initLatitude();
    longitude = initLongitude();
  }

  Map<String, double> initLatitude() {
    Map<String, double> initValue = {
      'Sapporo': mapSapporoLatitude,
      'Tokyo': mapTokyoLatitude,
      'Osaka': mapOsakaLatitude,
      'Asahikawa': mapAsahikawaLatitude,
      'Fukushima': mapFukushimaLatitude,
    };
    return initValue;
  }

  Map<String, double> initLongitude() {
    Map<String, double> initValue = {
      'Sapporo': mapSapporoLongitude,
      'Tokyo': mapTokyoLongitude,
      'Osaka': mapOsakaLongitude,
      'Asahikawa': mapAsahikawaLongitude,
      'Fukushima': mapFukushimaLongitude,
    };
    return initValue;
  }

  void _animatedZoom(LatLng destLocation, double zoom) {
    final latTween = Tween<double>(
        begin: _mapController.camera.center.latitude,
        end: destLocation.latitude);
    final lngTween = Tween<double>(
        begin: _mapController.camera.center.longitude,
        end: destLocation.longitude);
    final zoomTween = Tween(begin: _mapController.camera.zoom, end: zoom);
    final controller = AnimationController(
        duration: const Duration(milliseconds: mapAnimationDurationMs),
        vsync: this);
    final Animation<double> animation =
        CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn);

    controller.addListener(() {
      _mapController.move(
          LatLng(latTween.evaluate(animation), lngTween.evaluate(animation)),
          zoomTween.evaluate(animation));
    });

    animation.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        controller.dispose();
      } else if (status == AnimationStatus.dismissed) {
        controller.dispose();
      }
    });

    controller.forward();
  }

  Marker _createMarker(String label) {
    return Marker(
      point: LatLng(latitude[label]!, longitude[label]!),
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
          onTap: () {
            setState(() {
              pinSize.updateAll((key, value) => mapPinDefaultSize);
              pinSize[label] = mapPinFocusSize;
              // Icon is aligned in the upper left corner,
              // so if you increase the size, it will shift.
              latitude = initLatitude();
              longitude = initLongitude();
              latitude[label] = latitude[label]! + mapPinLatitudeSlide;
              longitude[label] = longitude[label]! + mapPinLongitudeSlide;
            });
          },
          child: Icon(
            Icons.location_pin,
            color: mapPointColor,
            size: pinSize[label],
          ),
        ),
      ),
    );
  }

  TileLayer _mapTileLayer() => TileLayer(
        urlTemplate: 'assets/map/{z}/{x}/{y}.png',
        tileProvider: AssetTileProvider(),
        tileBounds: LatLngBounds(LatLng(mapMinLatitude, mapMinLongitude),
            LatLng(mapMaxLatitude, mapMaxLongitude)),
      );

  @override
  Widget build(BuildContext context) {
    final Size containerSize = MediaQuery.of(context).size;
    return Scaffold(
      body: FlutterMap(
        mapController: _mapController,
        options: MapOptions(
            initialCenter: LatLng(mapCenterLatitude, mapCenterLongitude),
            initialZoom: mapMinZoom,
            cameraConstraint: CameraConstraint.containCenter(
              bounds: LatLngBounds(LatLng(mapMinLatitude, mapMinLongitude),
                  LatLng(mapMaxLatitude, mapMaxLongitude)),
            ),
            maxZoom: mapMaxZoom,
            minZoom: mapMinZoom,
            onLongPress: (tapPosition, point) {
              if (_mapController.camera.zoom != mapMinZoom)
                _animatedZoom(point, mapMinZoom);
            }),
        children: [
          _mapTileLayer(),
          MarkerLayer(
            markers: [
              _createMarker('Sapporo'),
              _createMarker('Tokyo'),
              _createMarker('Osaka'),
              _createMarker(
                'Asahikawa',
              ),
              _createMarker(
                'Fukushima',
              ),
            ],
          ),
          Align(
            alignment: Alignment.bottomRight,
            child: Text(
              "\u00a9 https://www.openstreetmap.org/copyright contributors",
              style: TextStyle(
                  fontSize: containerSize.width * mapCopyrightFontSizeRatio),
            ),
          )
        ],
      ),
    );
  }
}
