import 'dart:async';
import 'package:camera/camera.dart';
import 'package:video_player/video_player.dart';
import 'package:flutter/material.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:gui_app/const/dimens.dart';

const List<Widget> previews = <Widget>[
  Text('Movie'),
  Text('Camera'),
];

class Video extends StatefulWidget {
  const Video({super.key});
  @override
  _VideoState createState() {
    return _VideoState();
  }
}

class _VideoState extends State<Video>
    with WidgetsBindingObserver, TickerProviderStateMixin {
  CameraController? _cameraController;
  late VideoPlayerController _videoPlayerController;
  Future<void>? _initializeCameraController;
  final List<bool> _isSelectedMov = [true];
  final List<bool> _isSelectedCam = [false];

  bool _visible = true;
  bool _available_camera = true;
  List<DropdownMenuItem<CameraDescription>> _availableCameraList = [];

  @override
  void initState() {
    super.initState();
    _ambiguate(WidgetsBinding.instance)?.addObserver(this);

    initCamera();

    _videoPlayerController = VideoPlayerController.asset(
      'assets/video/factory.mp4',
      videoPlayerOptions: VideoPlayerOptions(mixWithOthers: true),
    );

    _videoPlayerController.addListener(() {
      setState(() {});
    });
    _videoPlayerController.setLooping(true);
    _videoPlayerController.initialize();
  }

  @override
  void dispose() {
    super.dispose();
    _cameraController?.dispose();
    _videoPlayerController.dispose();
  }

  void initCamera() async {
    final List<CameraDescription> cameras = await availableCameras();
    if (cameras.length == 0) {
      _available_camera = false;
      return;
    }
    for (var cam in cameras) {
      _availableCameraList.add(DropdownMenuItem<CameraDescription>(
          child: Text(cam.name.substring(15)), value: cam));
    }

    _cameraController = CameraController(
      cameras[0],
      ResolutionPreset.medium,
      enableAudio: false,
    );

    _initializeCameraController = _cameraController?.initialize();
  }

  Future<void> onNewCameraSelected(CameraDescription cameraDescription) async {
    final CameraController? oldController = _cameraController;
    if (oldController != null) {
      // `controller` needs to be set to null before getting disposed,
      // to avoid a race condition when we use the controller that is being
      // disposed. This happens when camera permission dialog shows up,
      // which triggers `didChangeAppLifecycleState`, which disposes and
      // re-creates the controller.
      _cameraController = null;
      await oldController.dispose();
    }

    final CameraController cameraController = CameraController(
      cameraDescription,
      ResolutionPreset.medium,
      enableAudio: false,
    );

    _cameraController = cameraController;

    _initializeCameraController = cameraController.initialize();

    if (mounted) {
      setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    final double deviceHeight = MediaQuery.of(context).size.height;
    final double titleHeight = deviceHeight * titleHeightRatio;
    final double videoHeight = (deviceHeight - outsidePadding * 2 - titleHeight) / 2;
    return Column(
      children: <Widget>[
        SizedBox(height: 10),
        Row(
          children: [
            Container(
              padding: EdgeInsets.zero,
              decoration: BoxDecoration(
                color: Color(0xFF374151),
                borderRadius: BorderRadius.all(Radius.circular(5)),
              ),
              child: ToggleButtons(
                isSelected: _isSelectedMov,
                onPressed: (int index) {
                  if (_isSelectedMov[index] && !_isSelectedCam[index]) {
                    return;
                  }
                  setState(() {
                    _isSelectedMov[index] = !_isSelectedMov[index];
                    _isSelectedCam[index] = !_isSelectedCam[index];
                    _visible = !_visible;
                    if (_visible) {
                      _videoPlayerController.play();
                    } else {
                      _videoPlayerController.pause();
                    }
                  });
                },
                children: [
                  Text(L10n.of(context)!.movie, style: TextStyle(fontSize: 15))
                ],
                borderRadius: const BorderRadius.all(Radius.circular(5)),
                selectedBorderColor: Color(0xFF1D4ED8),
                selectedColor: Colors.white,
                fillColor: Color(0xFF1D4ED8),
                color: Colors.white,
                constraints: const BoxConstraints(
                  minHeight: 30.0,
                  minWidth: 80.0,
                ),
              ),
            ),
            SizedBox(width: 10),
            Container(
              padding: EdgeInsets.zero,
              decoration: BoxDecoration(
                color: Color(0xFF374151),
                borderRadius: BorderRadius.all(Radius.circular(5)),
              ),
              child: ToggleButtons(
                isSelected: _isSelectedCam,
                onPressed: (int index) {
                  if (!_isSelectedMov[index] && _isSelectedCam[index]) {
                    return;
                  }
                  setState(() {
                    _isSelectedMov[index] = !_isSelectedMov[index];
                    _isSelectedCam[index] = !_isSelectedCam[index];
                    _visible = !_visible;
                    if (_visible) {
                      _videoPlayerController.play();
                    } else {
                      _videoPlayerController.pause();
                    }
                  });
                },
                children: [
                  Text(L10n.of(context)!.camera, style: TextStyle(fontSize: 15))
                ],
                borderRadius: const BorderRadius.all(Radius.circular(8)),
                selectedBorderColor: Color(0xFF1D4ED8),
                selectedColor: Colors.white,
                fillColor: Color(0xFF1D4ED8),
                color: Colors.white,
                constraints: const BoxConstraints(
                  minHeight: 30.0,
                  minWidth: 80.0,
                ),
              ),
            ),
          ],
        ),
        SizedBox(height: 10),
        Container(
          width: 512,
          height: videoHeight - 52,
          child: Center(
            child: Stack(
              alignment: AlignmentDirectional.center,
              children: <Widget>[
                FutureBuilder<void>(
                  future: _initializeCameraController,
                  builder: (context, snapshot) {
                    if (snapshot.connectionState == ConnectionState.done) {
                      return CameraPreview(_cameraController!);
                    } else {
                      if (_available_camera) {
                        return const Center(child: CircularProgressIndicator());
                      } else {
                        return const _NoCameraOverlay();
                      }
                    }
                  },
                ),
                Visibility(
                  visible: _visible,
                  maintainState: true,
                  child: Container(
                    decoration: BoxDecoration(color: Colors.white),
                    child: Stack(
                      children: <Widget>[
                        VideoPlayer(_videoPlayerController),
                        _ControlsOverlay(controller: _videoPlayerController),
                      ],
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }
}

class _ControlsOverlay extends StatelessWidget {
  const _ControlsOverlay({Key? key, required this.controller})
      : super(key: key);

  final VideoPlayerController controller;

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        AnimatedSwitcher(
          duration: const Duration(milliseconds: 50),
          reverseDuration: const Duration(milliseconds: 200),
          child: controller.value.isPlaying
              ? const SizedBox.shrink()
              : Container(
                  color: Colors.black26,
                  child: const Center(
                    child: Icon(
                      Icons.play_arrow,
                      color: Colors.white,
                      size: 100.0,
                    ),
                  ),
                ),
        ),
        GestureDetector(
          onTap: () {
            controller.value.isPlaying ? controller.pause() : controller.play();
          },
        ),
      ],
    );
  }
}

class _NoCameraOverlay extends StatelessWidget {
  const _NoCameraOverlay({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black26,
      child: const Center(
        child: Icon(
          Icons.no_photography,
          color: Colors.white,
          size: 100.0,
        ),
      ),
    );
  }
}

/// This allows a value of type T or T? to be treated as a value of type T?.
///
/// We use this so that APIs that have become non-nullable can still be used
/// with `!` and `?` on the stable branch.
// TODO(ianh): Remove this once we roll stable in late 2021.
T? _ambiguate<T>(T? value) => value;
